"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .transactionaction import TransactionAction, TransactionActionTypedDict
from gr4vy.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class TransactionActionsTypedDict(TypedDict):
    items: List[TransactionActionTypedDict]
    r"""The list of actions triggered for a transaction."""


class TransactionActions(BaseModel):
    items: List[TransactionAction]
    r"""The list of actions triggered for a transaction."""
