"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import UnrecognizedStr
from typing import Literal, Union


FlowAction = Union[
    Literal[
        "select-payment-options",
        "route-transaction",
        "decline-early",
        "skip-3ds",
    ],
    UnrecognizedStr,
]
