"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from typing_extensions import TypedDict


class PaypalShippingOptionsItemAmountTypedDict(TypedDict):
    currency_code: str
    r"""The three-character ISO currency code."""
    value: str
    r"""The amount value, which might include a decimal portion."""


class PaypalShippingOptionsItemAmount(BaseModel):
    currency_code: str
    r"""The three-character ISO currency code."""

    value: str
    r"""The amount value, which might include a decimal portion."""
