"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from typing_extensions import TypedDict


class MonatoSpeiOptionsTypedDict(TypedDict):
    approval_url: str
    r"""Approval URL that will receive a charge payment method reference."""


class MonatoSpeiOptions(BaseModel):
    approval_url: str
    r"""Approval URL that will receive a charge payment method reference."""
