"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .forterantifraudoptionscartitembasicitemdata import (
    ForterAntiFraudOptionsCartItemBasicItemData,
    ForterAntiFraudOptionsCartItemBasicItemDataTypedDict,
)
from .forterantifraudoptionscartitembeneficiary import (
    ForterAntiFraudOptionsCartItemBeneficiary,
    ForterAntiFraudOptionsCartItemBeneficiaryTypedDict,
)
from .forterantifraudoptionscartitemdeliverydetails import (
    ForterAntiFraudOptionsCartItemDeliveryDetails,
    ForterAntiFraudOptionsCartItemDeliveryDetailsTypedDict,
)
from gr4vy.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List
from typing_extensions import NotRequired, TypedDict


class ForterAntiFraudOptionsCartItemTypedDict(TypedDict):
    basic_item_data: NotRequired[
        Nullable[ForterAntiFraudOptionsCartItemBasicItemDataTypedDict]
    ]
    r"""Basic information about the cart item."""
    delivery_details: NotRequired[
        Nullable[ForterAntiFraudOptionsCartItemDeliveryDetailsTypedDict]
    ]
    r"""Details about how the item will be delivered."""
    beneficiaries: NotRequired[
        Nullable[List[ForterAntiFraudOptionsCartItemBeneficiaryTypedDict]]
    ]
    r"""List of beneficiaries who will receive this item."""


class ForterAntiFraudOptionsCartItem(BaseModel):
    basic_item_data: OptionalNullable[ForterAntiFraudOptionsCartItemBasicItemData] = (
        UNSET
    )
    r"""Basic information about the cart item."""

    delivery_details: OptionalNullable[
        ForterAntiFraudOptionsCartItemDeliveryDetails
    ] = UNSET
    r"""Details about how the item will be delivered."""

    beneficiaries: OptionalNullable[List[ForterAntiFraudOptionsCartItemBeneficiary]] = (
        UNSET
    )
    r"""List of beneficiaries who will receive this item."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["basic_item_data", "delivery_details", "beneficiaries"]
        nullable_fields = ["basic_item_data", "delivery_details", "beneficiaries"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m
