"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from typing_extensions import TypedDict


class ClickToPaySessionTypedDict(TypedDict):
    digital_payment_application_id: str
    r"""The ID of the Click to Pay application."""
    digital_payment_application_name: str
    r"""The merchant name as configured ont he the Click to Pay wallet."""


class ClickToPaySession(BaseModel):
    digital_payment_application_id: str
    r"""The ID of the Click to Pay application."""

    digital_payment_application_name: str
    r"""The merchant name as configured ont he the Click to Pay wallet."""
