"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gr4vy.types import BaseModel
from typing_extensions import TypedDict


class GiftCardStoredRequestTypedDict(TypedDict):
    r"""The ID of a stored gift card to fetch a balance for."""

    id: str
    r"""The ID of the field to fetch a balance for."""


class GiftCardStoredRequest(BaseModel):
    r"""The ID of a stored gift card to fetch a balance for."""

    id: str
    r"""The ID of the field to fetch a balance for."""
